@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -111,7 +111,7 @@ sub ldapCon
 
     if ((not defined $self->{ldap}) or $reconnect) {
         $self->{ldap} = $self->anonymousLdapCon();
-  
+
         my $global = EBox::Global->getInstance();
         my ($dn, $pass);
         my $auth_type = undef;
@@ -135,7 +135,7 @@ sub ldapCon
         } else {
             throw EBox::Exceptions::Internal("Unknown auth_type: $auth_type");
         }
-        $self->{ldap}->bind($dn, password => $pass);
+        safeBind($self->{ldap}, $dn, $pass);
     }
     return $self->{ldap};
 }
@@ -147,7 +147,7 @@ sub ldapCon
 #
 # Returns:
 #
-#       An object of class Net::LDAP 
+#       An object of class Net::LDAP
 #
 # Exceptions:
 #
@@ -155,28 +155,7 @@ sub ldapCon
 sub anonymousLdapCon
 {
     my ($self) = @_;
-    # We try to connect 5 times in 5 seconds, as we might need to
-    # give slapd some time to accept connections after a
-    # slapd restart
-
-    my $ldap;
-
-    my $connected = undef;
-    for (0..4) {
-        $ldap = Net::LDAP->new (LDAPI);
-        if ($ldap) {
-            $connected = 1;
-            last;
-        } else {
-            sleep (1);
-        }
-    }
-
-    unless ($connected) {
-        throw EBox::Exceptions::Internal("Can't create ldapi connection");
-    }
-
-
+    my $ldap = EBox::Ldap::safeConnect(LDAPI);
     return $ldap;
 }
 
@@ -257,6 +236,17 @@ sub dn
     return $self->{dn};
 }
 
+# Method: clearConn
+#
+#       Closes LDAP connection and clears DN cached value
+#
+sub clearConn
+{
+    my ($self) = @_;
+    delete $self->{dn};
+    delete $self->{ldap};
+}
+
 # Method: rootDn
 #
 #       Returns the dn of the priviliged user
@@ -297,7 +287,8 @@ sub rootPw
 #
 sub slapdConfFile
 {
-    return SLAPDCONFFILE;}
+    return SLAPDCONFFILE;
+}
 
 # Method: ldapConf
 #
@@ -693,7 +684,7 @@ sub lastModificationTime
     my $lastStamp = $sortedEntries[-1]->get_value('modifyTimestamp');
 
     # Convert to seconds since epoch
-    my ($year, $month, $day, $h, $m, $s) = 
+    my ($year, $month, $day, $h, $m, $s) =
       $lastStamp =~ /([0-9]{4})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})([0-9]{2})Z/;
     return POSIX::mktime( $s, $m, $h, $day, $month -1, $year - 1900 );
 
@@ -758,20 +749,14 @@ sub dataDir
 sub stop
 {
     my ($self) = @_;
-
     EBox::Sudo::root(INIT_SCRIPT . ' stop');
-
-    sleep 1;
     return  $self->refreshLdap();
 }
 
 sub  start
 {
     my ($self) = @_;
-
     EBox::Sudo::root(INIT_SCRIPT . ' start');
-
-    sleep 1;
     return  $self->refreshLdap();
 }
 
@@ -808,9 +793,10 @@ sub _dumpLdap
     my $ldifFile = $self->ldifFile($dir, $slapd, $type);
 
     my $slapcatCommand = $self->_slapcatCmd($ldifFile, $slapd, $type);
-    my $chownCommand = "/bin/chown $user.$group $ldifFile";
+    my $chownCommand = "/bin/chown $user:$group $ldifFile";
 
-    $self->_pauseAndExecute(cmds => [$slapcatCommand, $chownCommand]);
+    $self->_execute(1, # With pause
+        cmds => [$slapcatCommand, $chownCommand]);
 }
 
 sub _dumpLdapData
@@ -873,10 +859,9 @@ sub _loadLdap
     my $chownConfCommand = $self->_chownConfDir($slapd);
     my $chownDataCommand = $self->_chownDataDir($slapd);
 
-    $self->_execute(
-                cmds => [$backupCommand, $mkCommand, $rmCommand,
-                         $slapaddCommand, $chownConfCommand, $chownDataCommand
-                        ]);
+    $self->_execute(0, # Without pause
+        cmds => [$backupCommand, $mkCommand, $rmCommand, $slapaddCommand,
+            $chownConfCommand, $chownDataCommand]);
 }
 
 sub _loadLdapData
@@ -961,6 +946,46 @@ sub restoreLdapFrontend
     $self->_loadLdapData($dir, 'frontend');
 }
 
+sub usersInBackup
+{
+    my ($self, $dir, $mode) = @_;
+
+    if ($mode eq 'slave') {
+        EBox::error('Not implemented: slave mode precheck for LDAP backup');
+        return [];
+    }
+
+    my @users;
+
+    my $ldifFile = $self->ldifFile($dir, 'master', 'data');
+
+    my $ldif = Net::LDAP::LDIF->new($ldifFile, 'r', onerror => 'undef');
+    my $usersDn;
+
+    while (not $ldif->eof()) {
+        my $entry = $ldif->read_entry ( );
+        if ($ldif->error()) {
+           EBox::error("Error reading LDIOF file $ldifFile: " . $ldif->error() .
+                       '. Error lines: ' .  $ldif->error_lines());
+        } else {
+            my $dn = $entry->dn();
+            if (not defined $usersDn) {
+                # first entry, use it to fetch the DN
+                $usersDn = 'ou=Users,' . $dn;
+                next;
+            }
+
+            # in zentyal users are identified by DN, not by objectclass
+            if ($dn =~ /$usersDn$/) {
+                push @users, $entry->get_value('uid');
+            }
+        }
+    }
+    $ldif->done();
+
+    return \@users;
+}
+
 sub _chownConfDir
 {
     my ($self, $slapd) = @_;
@@ -1034,17 +1059,17 @@ sub _backupSystemDirectory
     return EBox::Config::share() . '/ebox-usersandgroups/slapd.backup';
 }
 
-sub _pauseAndExecute
+sub _execute
 {
-    my ($self, %params) = @_;
-    my @cmds = @{ $params{cmds}  };
+    my ($self, $pause, %params) = @_;
+    my @cmds = @{ $params{cmds} };
     my $onError = $params{onError};
 
-    $self->stop();
+    if ($pause) {
+        $self->stop();
+    }
     try {
-        foreach my $cmd (@cmds) {
-            EBox::Sudo::root($cmd);
-        }
+        EBox::Sudo::root(@cmds);
     }
     otherwise {
         my $ex = shift;
@@ -1056,30 +1081,70 @@ sub _pauseAndExecute
         throw $ex;
     }
     finally {
-        $self->start();
+        if ($pause) {
+            $self->start();
+        }
     };
 }
 
-sub _execute
+sub _tryToStartSlapd
 {
-    my ($self, %params) = @_;
-    my @cmds = @{ $params{cmds}  };
-    my $onError = $params{onError};
+    # FIXME: We need to know the real mode here, but as some models may
+    # require to ask for the mode when being setted up, we cannot use
+    # the model manager, we'd enter in infinite recursion.
 
-    try {
-        foreach my $cmd (@cmds) {
-            EBox::Sudo::root($cmd);
-        }
+    # my $users = EBox::Global->modInstance('users');
+    # my $mode = $users->mode();
+    #
+    # if ($mode ne 'slave') {
+    #    EBox::Sudo::root('/etc/init.d/slapd start');
+    #}
+
+    if (! -d '/etc/ldap/slapd-replica.d') { # not slave
+        EBox::Sudo::root('/etc/init.d/slapd start');
     }
-    otherwise {
-        my $ex = shift;
+}
 
-        if ($onError) {
-            $onError->($self);
-        }
+sub safeConnect
+{
+    my ($ldapurl) = @_;
+    my $retries = 4;
+    my $ldap;
 
-        throw $ex;
+    local $SIG{PIPE};
+    $SIG{PIPE} = sub {
+       EBox::warn('SIGPIPE received connecting to LDAP');
     };
+    while (not $ldap = Net::LDAP->new($ldapurl) and $retries--) {
+        _tryToStartSlapd();
+        EBox::error("Couldn't connect to LDAP server $ldapurl, retrying");
+        sleep(1);
+    }
+
+    unless ($ldap) {
+        throw EBox::Exceptions::Internal(
+            "FATAL: Couldn't connect to LDAP server: $ldapurl");
+    }
+
+    if ($retries < 3) {
+        EBox::info('LDAP reconnect successful');
+    }
+
+    return $ldap;
+}
+
+sub safeBind
+{
+    my ($ldap, $dn, $password) = @_;
+
+    my $bind = $ldap->bind($dn, password => $password);
+    unless ($bind->{resultCode} == 0) {
+        throw EBox::Exceptions::Internal(
+            'Couldn\'t bind to LDAP server, result code: ' .
+            $bind->{resultCode});
+    }
+
+    return $bind;
 }
 
 1;
