@@ -1,5 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
-# Copyright (C) 2009 eBox Technologies S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -26,6 +25,8 @@ use EBox::Ldap;
 
 use Error qw(:try);
 
+use constant ROOT_CONFIG_DN => 'cn=admin,cn=config';
+
 sub new
 {
 	my $class = shift;
@@ -36,8 +37,8 @@ sub new
 
 # Method: _ldapModImplementation
 #
-#      	All modules using any of the functions in LdapUserBase.pm
-#   	should override this method to return the implementation
+#	All modules using any of the functions in LdapUserBase.pm
+#	should override this method to return the implementation
 #	of that interface.
 #
 # Returns:
@@ -76,13 +77,13 @@ sub masterLdap
     } else {
         my $remote = $users->remoteLdap();
         my $password = $users->remotePassword();
-        $ldap = Net::LDAP->new("ldap://$remote");
-        $ldap->bind($self->ldap->rootDn(), password => $password);
+        $ldap = EBox::Ldap::safeConnect("ldap://$remote");
+        EBox::Ldap::safeBind($ldap, $self->ldap->rootDn(), $password);
     }
     return $ldap;
 }
 
-#   Method: _loadSchema
+# Method: _loadSchema
 #
 #      loads an LDAP schema from an LDIF file
 #
@@ -95,24 +96,23 @@ sub _loadSchema
 
     my $users = EBox::Global->modInstance('users');
 
-    unless ($users->mode() eq 'slave') {
+    my $mode = $users->mode();
+    if ($mode eq 'master' or $mode eq 'ad-slave') {
         $self->ldap->ldapCon();
         my $ldap = $self->ldap->{ldap};
         $self->_loadSchemaDirectory($ldap, $ldiffile);
-    } else {
+    } elsif ($mode eq 'slave') {
         my $password = $self->ldap->getPassword();
         my $ldap;
         my @ports = (389, 1389, 1390);
         for my $port (@ports) {
-            for (0..4) {
-                $ldap = Net::LDAP->new("127.0.0.1:$port");
-                last if defined($ldap);
-                sleep(1);
-            }
-            defined($ldap) or throw EBox::Exceptions::Internal("Can't connect to LDAP on port $port");
-            $ldap->bind('cn=admin,cn=config', 'password' => $password);
+            $ldap = EBox::Ldap::safeConnect("127.0.0.1:$port");
+            EBox::Ldap::safeBind($ldap, ROOT_CONFIG_DN, $password);
             $self->_loadSchemaDirectory($ldap, $ldiffile);
         }
+    } else {
+         throw EBox::Exceptions::Internal(
+            "Trying to load schema with unknown LDAP mode: $mode");
     }
 }
 
@@ -139,10 +139,10 @@ sub _loadSchemaDirectory
             'attrs' => ['objectClass']
         );
         my $result = $ldap->search(%args);
-        if ($result->entries eq 0) {
+        if ($result->entries() == 0) {
             $result = $ldap->add($entry);
             if ($result->is_error()) {
-                EBox::debug($result->error());
+                EBox::error($result->error());
             }
         }
     }
@@ -161,25 +161,24 @@ sub _loadACL
     my ($self, $acl) = @_;
 
     my $users = EBox::Global->modInstance('users');
+    my $mode = $users->mode();
 
-    unless ($users->mode() eq 'slave') {
+    if ($mode eq 'master' or $mode eq 'ad-slave') {
         $self->ldap->ldapCon();
         my $ldap = $self->ldap->{ldap};
         $self->_loadACLDirectory($ldap, $acl);
-    } else {
+    } elsif ($mode eq 'slave') {
         my $password = $self->ldap->getPassword();
         my $ldap;
         my @ports = (389, 1389, 1390);
         for my $port (@ports) {
-            for (0..4) {
-                $ldap = Net::LDAP->new("127.0.0.1:$port");
-                last if defined($ldap);
-                sleep(1);
-            }
-            defined($ldap) or throw EBox::Exceptions::Internal("Can't connect to LDAP on port $port");
-            $ldap->bind('cn=admin,cn=config', 'password' => $password);
+            $ldap = EBox::Ldap::safeConnect("127.0.0.1:$port");
+            EBox::Ldap::safeBind($ldap, ROOT_CONFIG_DN, $password);
             $self->_loadACLDirectory($ldap, $acl);
         }
+    } else {
+        throw EBox::Exceptions::Internal(
+            "Loading ACL with unknown LDAP mode: $mode");
     }
 }
 
@@ -236,6 +235,79 @@ sub _addTranslucentLocalAttribute
     EBox::Sudo::root("sed -i -e 's/^olcTranslucentLocal: \\(.*\\)/olcTranslucentLocal: \\1,$attribute/' /etc/ldap/slapd-translucent.d/cn=config/olcDatabase={1}hdb/olcOverlay={0}translucent.ldif");
 }
 
+
+#   Method: _addIndex
+#
+#       Create indexes in LDAP for an attribute
+#
+# Parameters:
+#          attribute - string with the attribute to be indexed in LDAP
+#
+sub _addIndex
+{
+    my ($self, $attribute) = @_;
+
+    my $users = EBox::Global->modInstance('users');
+    my $mode = $users->mode();
+
+    if ($mode eq 'master' or $mode eq 'ad-slave') {
+        $self->ldap->ldapCon();
+        my $ldap = $self->ldap->{ldap};
+        $self->_addIndexDirectory($ldap, $attribute);
+    } elsif ($mode eq 'slave') {
+        my $password = $self->ldap->getPassword();
+        my $ldap;
+        my @ports = (389, 1389, 1390);
+        for my $port (@ports) {
+            $ldap = EBox::Ldap::safeConnect("127.0.0.1:$port");
+            EBox::Ldap::safeBind($ldap, ROOT_CONFIG_DN, $password);
+            $self->_addIndexDirectory($ldap, $attribute);
+        }
+    } else {
+        throw EBox::Exceptions::Internal(
+            "Creating index with unknown LDAP mode: $mode");
+    }
+}
+
+
+sub _addIndexDirectory
+{
+    my ($self, $ldap, $attribute) = @_;
+
+    my $index = "$attribute eq";
+
+    my $dn = 'olcDatabase={1}hdb,cn=config';
+    my %args = (
+            'base' => $dn,
+            'scope' => 'base',
+            'filter' => "(objectClass=*)",
+            'attrs' => ['olcDbIndex']
+    );
+    my $result = $ldap->search(%args);
+    my $entry = ($result->entries)[0];
+    my $attr = ($entry->attributes)[0];
+    my $found = undef;
+    my @indexes = $entry->get_value($attr);
+    for my $dbindex (@indexes) {
+        if($dbindex eq $index) {
+            $found = 1;
+            last;
+        }
+    }
+    if(not $found) {
+        push(@indexes, $index);
+        my %args = (
+            'replace' => [ 'olcDbIndex' => \@indexes ]
+        );
+        try {
+            $ldap->modify($dn, %args);
+        } otherwise {
+            throw EBox::Exceptions::Internal("Invalid index: $index");
+        };
+    }
+}
+
+
 #   Method: performLDAPActions
 #
 #      adds the schemas, acls and local attributes specified in the
@@ -262,6 +334,10 @@ sub performLDAPActions
     for my $acl (@acls) {
         $self->_loadACL($acl);
     }
+    my @indexes = @{ $ldapuser->indexes() };
+    for my $index (@indexes) {
+        $self->_addIndex($index);
+    }
     if ($slave) {
         $users->stopIfRequired();
         my @attrs = @{ $ldapuser->localAttributes() };
