@@ -1,4 +1,4 @@
-# Copyright
+# Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -31,13 +31,13 @@ use base 'EBox::Model::DataForm';
 
 sub new
 {
-        my $class = shift;
-        my %parms = @_;
+    my $class = shift;
+    my %parms = @_;
 
-        my $self = $class->SUPER::new(@_);
-        bless($self, $class);
+    my $self = $class->SUPER::new(@_);
+    bless($self, $class);
 
-        return $self;
+    return $self;
 }
 
 sub pageTitle
@@ -47,7 +47,6 @@ sub pageTitle
 
 sub _table
 {
-
     my @tableHead =
     (
         new EBox::Types::Port(
@@ -117,4 +116,24 @@ sub validateTypedRow
     }
 }
 
+# Method: precondition
+#
+#   Overrides  <EBox::Model::DataTable::precondition>
+#
+sub precondition
+{
+    my $users = EBox::Global->modInstance('users');
+
+    return $users->editableMode();
+}
+
+# Method: preconditionFailMsg
+#
+#   Overrides  <EBox::Model::DataTable::preconditionFailMsg>
+#
+sub preconditionFailMsg
+{
+    return __('User Corner is not supported on slave servers.');
+}
+
 1;
