@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -30,7 +30,7 @@ use EBox::Ldap;
 use EBox::Gettext;
 use EBox::Menu::Folder;
 use EBox::Menu::Item;
-use EBox::Sudo qw( :all );
+use EBox::Sudo;
 use EBox::FileSystem;
 use EBox::LdapUserImplementation;
 use EBox::Config;
@@ -42,13 +42,14 @@ use Digest::SHA1;
 use Digest::MD5;
 use Crypt::SmbHash;
 use Sys::Hostname;
-use Net::LDAP;
 
 use Error qw(:try);
 use File::Copy;
 use File::Slurp;
 use File::Temp qw/tempfile/;
 use Perl6::Junction qw(any);
+use String::ShellQuote;
+use Fcntl qw(:flock);
 
 use constant USERSDN        => 'ou=Users';
 use constant GROUPSDN       => 'ou=Groups';
@@ -58,10 +59,11 @@ use constant SYSMINUID      => 1900;
 use constant SYSMINGID      => 1900;
 use constant MINUID         => 2000;
 use constant MINGID         => 2000;
-use constant HOMEPATH       => '/nonexistent';
-use constant MAXUSERLENGTH  => 32;
-use constant MAXGROUPLENGTH => 32;
+use constant HOMEPATH       => '/home';
+use constant MAXUSERLENGTH  => 128;
+use constant MAXGROUPLENGTH => 128;
 use constant MAXPWDLENGTH   => 512;
+use constant LIBNSSLDAPFILE => '/etc/ldap.conf';
 use constant SECRETFILE     => '/etc/ldap.secret';
 use constant LDAPCONFDIR    => '/etc/ldap/';
 use constant DEFAULTGROUP   => '__USERS__';
@@ -69,6 +71,9 @@ use constant CONFLDIF       => '/etc/ldap/eboxldap.ldif';
 use constant CA_DIR         => EBox::Config::conf() . 'ssl-ca/';
 use constant SSL_DIR        => EBox::Config::conf() . 'ssl/';
 use constant CERT           => SSL_DIR . 'master.cert';
+use constant AUTHCONFIGTMPL => '/etc/auth-client-config/profile.d/acc-ebox';
+use constant LOCK_FILE      => EBox::Config::tmp() . 'ebox-users-lock';
+
 
 sub _create
 {
@@ -88,47 +93,61 @@ sub _create
 #
 sub actions
 {
+    my ($self) = @_;
+
     my $mode = mode();
+    my @actions;
 
     if ($mode eq 'master') {
-        return [
+        push(@actions,
                 {
-                 'action' => __('Your openLDAP database will be populated with some basic organizational units'),
-                 'reason' => __('eBox needs this organizational units to add users and groups into them'),
+                 'action' => __('Your LDAP database will be populated with some basic organizational units'),
+                 'reason' => __('Zentyal needs this organizational units to add users and groups into them.'),
                  'module' => 'users'
-            },
-           {
-            'action' => __(q{Create directories for slave's journals}),
-            'reason' => __(q{eBox needs the directories to record pending slave's actions}),
-            'module' => 'users'
-            }
-        ];
-    } elsif ($mode eq 'slave') {
-        my @actions = (
+                },
                 {
-                 'action' => __('Your eBox will be registered as a slave in the eBox master specified'),
-                 'reason' => __('This eBox needs to have remote access to the users in the eBox master'),
-                 'module' => 'users'
-            }
-        );
-        if ( -f '/etc/init.d/apparmor' ) {
-            push (@actions, {
-               'action' => __('Apparmor profile will be disabled'),
-               'reason' => __('It is not ready to work with more than one slapd'),
-               'module' => 'users'
-            });
-        }
-        return \@actions;
-    } elsif ($mode eq 'ad-slave') {
-        return [
-                {
-                 'action' => __('Install /etc/cron.d/ebox-ad-sync.'),
-                 'reason' => __('eBox will run a script each 5 minutes to sync with Windows AD.'),
+                 'action' => __('Create directories for slave journals'),
+                 'reason' => __('Zentyal needs the directories to record pending slave actions.'),
                  'module' => 'users'
                 }
-        ];
+        );
+    } elsif ($mode eq 'slave') {
+        push(@actions,
+                {
+                 'action' => __('Your Zentyal will be registered as a slave in the Zentyal master specified'),
+                 'reason' => __('This Zentyal needs to have remote access to the users in the Zentyal master.'),
+                 'module' => 'users'
+                }
+        );
+        if ( -f '/etc/init.d/apparmor' ) {
+            push(@actions,
+                    {
+                     'action' => __('Apparmor profile will be disabled'),
+                     'reason' => __('It is not ready to work with more than one slapd.'),
+                     'module' => 'users'
+                    }
+            );
+        }
+    } elsif ($mode eq 'ad-slave') {
+        push(@actions,
+                {
+                 'action' => __('Install /etc/cron.d/ebox-ad-sync.'),
+                 'reason' => __('Zentyal will run a script every 5 minutes to sync with Windows AD.'),
+                 'module' => 'users'
+                }
+        );
 
     }
+    if ($self->model('PAM')->enable_pamValue()) {
+        push(@actions,
+                {
+                 'action' => __('Configure PAM.'),
+                 'reason' => __('Zentyal will give LDAP users system account.'),
+                 'module' => 'users'
+                }
+        );
+    }
+    return \@actions;
 }
 
 # Method: usedFiles
@@ -137,25 +156,40 @@ sub actions
 #
 sub usedFiles
 {
+    my @files = ();
     my $mode = mode();
 
+    push(@files,
+            {
+             'file' => '/etc/nsswitch.conf',
+             'reason' => __('To make NSS use LDAP resolution for user and group '.
+                 'accounts. Needed for Samba PDC configuration.'),
+             'module' => 'samba'
+            },
+            {
+             'file' => LIBNSSLDAPFILE,
+             'reason' => __('To let NSS know how to access LDAP accounts.'),
+             'module' => 'samba'
+            },
+    );
+
     if ($mode ne 'slave') {
-        return [
+        push(@files,
                 {
                  'file' => '/etc/default/slapd',
-                 'reason' => __('To make openLDAP listen on TCP and Unix sockets'),
+                 'reason' => __('To make LDAP listen on TCP and Unix sockets.'),
                  'module' => 'users'
                 },
                 {
-                    'file' => '/etc/ldap.secret',
+                    'file' => SECRETFILE,
                     'reason' => __('To copy LDAP admin password generated by ' .
-                        ' eBox and allow other modules to access LDAP'),
+                        'Zentyal and allow other modules to access LDAP.'),
                     'module' => 'users'
                 },
-        ];
-    } else {
-        return [];
+        );
     }
+
+    return \@files;
 }
 
 # Method: enableActions
@@ -165,6 +199,14 @@ sub usedFiles
 sub enableActions
 {
     my ($self) = @_;
+    my $share = EBox::Config::share();
+
+    # Lock to operate in exclusive mode
+    open(my $lock, '>', LOCK_FILE);
+    flock($lock, LOCK_EX);
+
+    return if ($self->{enabled});
+    $self->{enabled} = 1;
 
     my $mode = mode();
 
@@ -172,18 +214,40 @@ sub enableActions
         $self->disableApparmorProfile('usr.sbin.slapd');
 
         EBox::Sudo::root("invoke-rc.d slapd stop");
-        EBox::Sudo::root("cp " . EBox::Config::share() . "/ebox-usersandgroups/slapd.default.no /etc/default/slapd");
+        EBox::Sudo::root("cp $share/ebox-usersandgroups/slapd.default.no " .
+            '/etc/default/slapd');
 
         $self->_setupSlaveLDAP();
-    } else { # Same setup for 'master' and 'ad-slave' modes
+    } elsif ($mode eq 'master' or $mode eq 'ad-slave') {
         my $password = remotePassword();
 
         EBox::UsersAndGroups::Setup::master($password);
 
         $self->performLDAPActions();
+    } else {
+        throw EBox::Exceptions::Internal(
+            "Trying to enable users with unknown LDAP mode: $mode");
     }
+    EBox::Sudo::root("$share/ebox-usersandgroups/ebox-usersandgroups-enable");
+
+    # Release lock
+    flock($lock, LOCK_UN);
+    close($lock);
 }
 
+
+# Method: wizardPages
+#
+#   Override EBox::Module::Base::wizardPages
+#
+sub wizardPages
+{
+    my ($self) = @_;
+
+    return [ '/UsersAndGroups/Wizard/Users' ];
+}
+
+
 # Method: _setConf
 #
 #       Override EBox::Module::Service::_setConf
@@ -193,7 +257,6 @@ sub _setConf
     my ($self) = @_;
 
     my $mode = mode();
-
     my $ldap = $self->ldap;
     EBox::Module::Base::writeFile(SECRETFILE, $ldap->getPassword(),
         { mode => '0600', uid => 0, gid => 0 });
@@ -211,8 +274,75 @@ sub _setConf
     }
 
     if ( $mode eq 'ad-slave' ) {
-        my $cronFile = EBox::Config::share() . '/ebox-usersandgroups/ebox-ad-sync.cron';
-        EBox::Sudo::root("install -m 0644 -o root -g root $cronFile /etc/cron.d/ebox-ad-sync");
+        EBox::Sudo::root("rm -f /etc/cron.d/ebox-ad-sync");
+        if ($self->adsyncEnabled()) {
+            my $cronFile = EBox::Config::share() . '/ebox-usersandgroups/ebox-ad-sync.cron';
+            EBox::Sudo::root("install -m 0644 -o root -g root $cronFile /etc/cron.d/ebox-ad-sync");
+        }
+    }
+
+    my @array = ();
+    my $dn;
+    if ( $mode eq 'slave' ) {
+        my $enablePam = $self->model('PAM')->enable_pamValue();
+
+        # Bind to translucent if PAM enabled, to frontend if not (necessary for samba)
+        if ( $enablePam ) {
+            push(@array, 'ldap' => 'ldap://127.0.0.1:1389');
+        } else {
+            push(@array, 'ldap' => 'ldap://127.0.0.1');
+        }
+        $dn = $self->model('Mode')->dnValue();
+    } else {
+        # master or ad-sync
+        push(@array, 'ldap' => EBox::Ldap::LDAPI);
+        $dn = $ldap->dn;
+    }
+
+    push(@array, 'basedc'    => $dn);
+    push(@array, 'binddn'    => 'cn=ebox,' . $dn);
+    push(@array, 'usersdn'   => USERSDN . ',' . $dn);
+    push(@array, 'groupsdn'  => GROUPSDN . ',' . $dn);
+    push(@array, 'computersdn' => 'ou=Computers,' . $dn);
+
+    $self->writeConfFile(LIBNSSLDAPFILE, "usersandgroups/ldap.conf.mas",
+            \@array);
+
+    $self->_setupNSSPAM();
+}
+
+# Method: adsyncEnabled
+#
+#       Returns true if ad-sync is enabled in ad-slave mode.
+#
+sub adsyncEnabled
+{
+    my ($self) = @_;
+
+    my $model = $self->model('ADSyncSettings');
+    return $model->enableADsyncValue();
+}
+
+
+# Method: editableMode
+#
+#       Check if users and groups can be edited.
+#
+#       Returns true if mode is master or disabled ad-sync
+#       Returns false if slave or enabled ad-sync
+#
+sub editableMode
+{
+    my ($self) = @_;
+
+    my $mode = $self->mode();
+
+    if ($mode eq 'master') {
+        return 1;
+    } elsif ($mode eq 'slave') {
+        return 0;
+    } elsif ($mode eq 'ad-slave') {
+        return not $self->adsyncEnabled();
     }
 }
 
@@ -222,6 +352,8 @@ sub _setConf
 #
 sub _daemons
 {
+    my ($self) = @_;
+
     my $mode = mode();
 
     if ($mode eq 'master') {
@@ -234,7 +366,10 @@ sub _daemons
         ];
     } elsif ($mode eq 'ad-slave') {
         return [
-            { 'name' => 'ebox.ad-pwdsync' },
+                {
+                  'name' => 'ebox.ad-pwdsync',
+                  'precondition' => \&adsyncEnabled
+                }
         ];
     }
 }
@@ -249,15 +384,21 @@ sub _enforceServiceState
 
     my $mode = mode();
 
-    if ($mode eq 'master') {
-        $self->_loadCertificates();
-    } else {
-        $self->SUPER::_enforceServiceState();
+    # FIXME: This method should not be overrided
+    # the good way to do this would be to have
+    # _preService and _postService methods in
+    # EBox::Module::Service like the _preSetConf
+    # and _postSetConf in EBox::Module::Base
 
-        if ($mode eq 'slave') {
-            my ($ldap, $dn) = $self->_connRemoteLDAP();
-            $self->_getCertificates($ldap, $dn);
-        }
+    if ($mode ne 'slave') {
+        $self->_loadCertificates();
+    }
+
+    $self->SUPER::_enforceServiceState();
+
+    if ($mode eq 'slave') {
+        my ($ldap, $dn) = $self->_connRemoteLDAP();
+        $self->_getCertificates($ldap, $dn);
     }
 }
 
@@ -272,15 +413,15 @@ sub _loadCertificates
     $cert = read_file(SSL_DIR . "ssl.cert");
     $ca = $cert;
 
-    EBox::Sudo::root('chown -R ebox.ebox /etc/ldap/ssl');
+    EBox::Sudo::root('chown -R ebox:ebox /etc/ldap/ssl');
     $ldapca = read_file("/etc/ldap/ssl/ssl.cert");
-    EBox::Sudo::root('chown -R openldap.openldap /etc/ldap/ssl');
+    EBox::Sudo::root('chown -R openldap:openldap /etc/ldap/ssl');
 
     my $dn = $self->masterDn();
 
     # delete old certs and add new ones
     try {
-    $self->ldap->delete($dn);
+        $self->ldap->delete($dn);
     } otherwise {};
 
     my %args = ('attr' => [
@@ -304,9 +445,10 @@ sub modelClasses
         'EBox::UsersAndGroups::Model::Groups',
         'EBox::UsersAndGroups::Model::Password',
         'EBox::UsersAndGroups::Model::Slaves',
-        'EBox::UsersAndGroups::Model::LdapInfo',
         'EBox::UsersAndGroups::Model::PendingSync',
         'EBox::UsersAndGroups::Model::ForceSync',
+        'EBox::UsersAndGroups::Model::LdapInfo',
+        'EBox::UsersAndGroups::Model::PAM',
         'EBox::UsersAndGroups::Model::ADSyncSettings',
     ];
 }
@@ -318,6 +460,7 @@ sub modelClasses
 sub compositeClasses
 {
     return [
+        'EBox::UsersAndGroups::Composite::Settings',
         'EBox::UsersAndGroups::Composite::SlaveInfo',
         'EBox::UsersAndGroups::Composite::UserTemplate',
     ];
@@ -343,7 +486,7 @@ sub groupsDn
 
 # Method: masterDn
 #
-#       Returns the dn where the eBox master machine is stored in LDAP
+#       Returns the dn where the Zentyal master machine is stored in LDAP
 #
 # Returns:
 #
@@ -358,7 +501,7 @@ sub masterDn
 
 # Method: slavesDn
 #
-#       Returns the dn where the eBox slave machiens are stored in LDAP
+#       Returns the dn where the Zentyal slave machiens are stored in LDAP
 #
 # Returns:
 #
@@ -591,8 +734,10 @@ sub rewriteObjectClasses
 {
     my ($self, $dn) = @_;
 
-    my $ldap = Net::LDAP->new("ldap://127.0.0.1:1389");
-    $ldap->bind($self->ldap->rootDn(), password => remotePassword());
+    my $ldap = EBox::Ldap::safeConnect('ldap://127.0.0.1:1389');
+    EBox::Ldap::safeBind(
+        $ldap, $self->ldap->rootDn(), remotePassword());
+
     my %attrs = (
             base   => $dn,
             filter => "(objectclass=*)",
@@ -615,8 +760,8 @@ sub rewriteObjectClasses
 sub rewriteObjectClassesTree
 {
     my ($self, $dn) = @_;
-    my $ldap = Net::LDAP->new("ldap://127.0.0.1:1389");
-    $ldap->bind($self->ldap->rootDn(), password => remotePassword());
+    my $ldap = EBox::Ldap::safeConnect('ldap://127.0.0.1:1389');
+    EBox::Ldap::safeBind($ldap, $self->ldap->rootDn(), remotePassword());
     my %attrs = (
             base   => $dn,
             filter => "(objectclass=*)",
@@ -633,11 +778,29 @@ sub initUser
 {
     my ($self, $user, $password) = @_;
 
+    my $home = $self->userInfo($user)->{'homeDirectory'};
+    if ($home and ($home ne '/dev/null') and (not -e $home)) {
+        my @cmds;
+
+        my $quser = shell_quote($user);
+        my $qhome = shell_quote($home);
+        my $group = DEFAULTGROUP;
+        push(@cmds, "mkdir -p `dirname $qhome`");
+        push(@cmds, "cp -dR --preserve=mode /etc/skel $qhome");
+        push(@cmds, "chown -R $quser:$group $qhome");
+
+        my $dir_umask = oct(EBox::Config::configkey('dir_umask'));
+        my $perms = sprintf("%#o", 00777 &~ $dir_umask);
+        push(@cmds, "chmod $perms $qhome");
+
+        EBox::Sudo::root(@cmds);
+    }
+
     # Tell modules depending on users and groups
     # a new new user is created
     my @mods = @{$self->_modsLdapUserBase()};
 
-    foreach my $mod (@mods){
+    foreach my $mod (@mods) {
         $mod->_addUser($user, $password);
     }
 }
@@ -665,14 +828,16 @@ sub soapRun
     my ($self, $slave, $method, $param, @params) = @_;
 
     my $journaldir = $self->_journalsDir . $slave->{'hostname'};
-    (-d $journaldir) or EBox::Sudo::command("mkdir -p $journaldir");
-
+    unless (-d $journaldir) {
+        EBox::Sudo::root('mkdir -p ' . $journaldir,
+                         'chown -R ebox:ebox ' . $self->_journalsDir);
+    }
     my $client = $self->soapClient($slave);
 
     try {
         $client->$method($param, @params);
     } otherwise {
-        EBox::debug("Unable to perform operation $method with parameter $param on slave $slave");
+        EBox::debug("Unable to perform operation $method with parameter $param on slave $slave->{'hostname'}");
         my ($fh, $filename) = tempfile("$method-XXXX", DIR => $journaldir);
         print $fh "$method\n";
         print $fh "$param\n";
@@ -729,6 +894,11 @@ sub addUser # (user, system)
                                          __x("Username must not be longer than {maxuserlength} characters",
                            maxuserlength => MAXUSERLENGTH));
     }
+    if (getpwnam($user->{'user'})) {
+        throw EBox::Exceptions::External(
+            __("Username already exists on the system")
+        );
+    }
     unless (_checkName($user->{'user'})) {
         throw EBox::Exceptions::InvalidData(
                                             'data' => __('user name'),
@@ -749,6 +919,9 @@ sub addUser # (user, system)
     my $gid = $self->groupGid(DEFAULTGROUP);
 
     my $passwd = $user->{'password'};
+    if (not $passwd) {
+        throw EBox::Exceptions::MissingArgument(__('Password'));
+    }
     if (not isHashed($passwd)) {
         $passwd =  defaultPasswordHash($passwd);
     }
@@ -785,16 +958,15 @@ sub addUser # (user, system)
         'cn'            => $user->{'fullname'},
         'uid'           => $user->{'user'},
         'sn'            => $user->{'surname'},
+        'loginShell'    => $self->_loginShell(),
         'uidNumber'     => $uid,
         'gidNumber'     => $gid,
-        'homeDirectory' => HOMEPATH,
+        'homeDirectory' => _homeDirectory($user->{'user'}),
         'userPassword'  => $passwd,
         'objectclass'   => ['inetOrgPerson', 'posixAccount', 'passwordHolder'],
         @additionalPasswords
     );
 
-
-
     my %args = ( attr => \@attr );
 
     my $dn = "uid=" . $user->{'user'} . "," . $self->usersDn;
@@ -1008,7 +1180,7 @@ sub _cleanUser
 
     # Tell modules depending on users and groups
     # an user is to be deleted
-    foreach my $mod (@mods){
+    foreach my $mod (@mods) {
         $mod->_delUser($user);
     }
 }
@@ -1485,7 +1657,9 @@ sub updateGroup
     my ($self, $group, @params) = @_;
 
     $self->updateGroupLocal($group, @params);
-    $self->_updateGroupSlaves($group, @params);
+    if (mode() ne 'slave') {
+        $self->_updateGroupSlaves($group, @params);
+    }
 }
 
 sub updateGroupLocal
@@ -1572,7 +1746,7 @@ sub delGroup # (group)
     my ($self, $group) = @_;
 
     unless ($self->groupExists($group)) {
-        throw EBox::Exceptions::DataNotFoud('data' => __('group name'),
+        throw EBox::Exceptions::DataNotFound('data' => __('group name'),
                                             'value' => $group);
     }
 
@@ -1609,7 +1783,7 @@ sub delGroupSlave # (group)
 #
 # Returns:
 #
-#       hash ref - holding the keys: 'groupname' and 'description'
+#       hash ref - holding the keys: 'groupname', 'comment' and 'gid'
 sub groupInfo # (group)
 {
     my ($self, $group) = @_;
@@ -2061,7 +2235,7 @@ sub _checkName
 {
     my ($name) = @_;
 
-    if ($name =~ /^[a-zA-Z\d\s_-]+\.?[a-zA-Z\d\s_-]+$/) {
+    if ($name =~ /^([a-zA-Z\d\s_-]+\.)*[a-zA-Z\d\s_-]+$/) {
         return 1;
     } else {
         return undef;
@@ -2268,33 +2442,34 @@ sub menu
         my $model = EBox::Model::ModelManager->instance()->model('Mode');
         my $mode = $model->modeValue();
 
-        if ($mode eq 'master') {
+        if ($self->editableMode()) {
             $folder->add(new EBox::Menu::Item('url' => 'UsersAndGroups/Users',
                                               'text' => __('Users')));
             $folder->add(new EBox::Menu::Item('url' => 'UsersAndGroups/Groups',
                                               'text' => __('Groups')));
             $folder->add(new EBox::Menu::Item('url' => 'Users/Composite/UserTemplate',
-                                              'text' => __('Default User Template')));
+                                              'text' => __('User Template')));
         } else {
             $folder->add(new EBox::Menu::Item(
-                        'url' => '/Users/View/Users',
+                        'url' => 'Users/View/Users',
                         'text' => __('Users')));
             $folder->add(new EBox::Menu::Item(
-                        'url' => '/Users/View/Groups',
+                        'url' => 'Users/View/Groups',
                         'text' => __('Groups')));
             $folder->add(new EBox::Menu::Item('url' => 'Users/Composite/UserTemplate',
-                                              'text' => __('Default User Template')));
+                                              'text' => __('User Template')));
         }
-        if (($mode eq 'master') or ($mode eq 'ad-slave')) {
-            $folder->add(new EBox::Menu::Item(
-                'url' => '/Users/View/LdapInfo',
-                'text' => __('LDAP Info')));
-        }
-        if ($mode eq 'master') {
+
+        $folder->add(new EBox::Menu::Item(
+                    'url' => 'Users/Composite/Settings',
+                    'text' => __('LDAP Settings')));
+
+        if ($mode eq 'master' or $mode eq 'ad-slave') {
             $folder->add(new EBox::Menu::Item(
                         'url' => 'Users/Composite/SlaveInfo',
                         'text' => __('Slave Status')));
-        } elsif ($mode eq 'ad-slave') {
+        }
+        if ($mode eq 'ad-slave') {
             $folder->add(new EBox::Menu::Item(
                         'url' => 'Users/View/ADSyncSettings',
                         'text' => __('AD Sync Settings')));
@@ -2332,7 +2507,7 @@ sub dumpConfig
 
     my $mode = mode();
 
-    if ($mode eq 'master') {
+    if ($mode eq 'master' or $mode eq 'ad-slave') {
         $self->ldap->dumpLdapMaster($dir);
         if ($options{bug}) {
             my $file = $self->ldap->ldifFile($dir, 'master', 'data');
@@ -2342,32 +2517,107 @@ sub dumpConfig
         $self->ldap->dumpLdapReplica($dir);
         $self->ldap->dumpLdapTranslucent($dir);
         $self->ldap->dumpLdapFrontend($dir);
+    } else {
+        throw EBox::Exceptions::Internal(
+            "Trying to dump configuration of unknown LDAP mode: $mode");
     }
 }
 
+sub _modeToBeRestored
+{
+    my ($self, $dir) = @_;
+
+    my $masterFile = $self->ldap()->ldifFile($dir, 'master', 'data');
+    if (-r $masterFile ) {
+        # master or ad-slave is the same
+        return 'master';
+    } else {
+        return 'slave';
+    }
+}
+
+sub _usersInEtcPasswd
+{
+    my ($self) = @_;
+    my @users;
+
+    my @lines = File::Slurp::read_file('/etc/passwd');
+    foreach my $line (@lines) {
+        my ($user) = split ':', $line, 2;
+        push @users, $user;
+    }
+
+    return \@users;
+}
+
+sub restoreBackupPreCheck
+{
+    my ($self, $dir) = @_;
+
+    # get what will be the mode to be restored
+    my $mode = $self->_modeToBeRestored($dir);
+    if ($mode eq 'slave') {
+        # TODO: implement check for slave setups
+        return;
+    }
+
+    my %etcPasswdUsers = map { $_ => 1 } @{ $self->_usersInEtcPasswd() };
+
+    my @usersToRestore = @{ $self->ldap->usersInBackup($dir, $mode) };
+    foreach my $user (@usersToRestore) {
+        if (exists $etcPasswdUsers{$user}) {
+            throw EBox::Exceptions::External(
+                                             __x(
+'Cannot restore because user {user} already exists as system user. Delete or rename this user and try again',
+                                                 user => $user
+                                                )
+                                            );
+        }
+    }
+}
 
 sub restoreConfig
 {
     my ($self, $dir) = @_;
+    my $mode = mode();
+
+    if ($mode eq 'master' or $mode eq 'ad-slave') {
+        EBox::UsersAndGroups::Setup::createDefaultGroupIfNeeded();
 
-    unless (mode() eq 'slave') {
         EBox::Sudo::root('/etc/init.d/slapd stop');
         $self->ldap->restoreLdapMaster($dir);
         EBox::Sudo::root('/etc/init.d/slapd start');
+        $self->ldap->clearConn();
+
+        # Save conf to enable NSS (and/or) PAM
+        $self->_setConf();
         for my $user ($self->users()) {
             $self->initUser($user->{'username'});
         }
-    } else {
+    } elsif ($mode eq 'slave') {
         $self->_manageService('stop');
         $self->ldap->restoreLdapReplica($dir);
         $self->ldap->restoreLdapTranslucent($dir);
         $self->ldap->restoreLdapFrontend($dir);
+        $self->ldap->clearConn();
         $self->_manageService('start');
-        $self->waitSync();
+        try {
+            $self->waitSync();
+        } otherwise {
+            my $model = EBox::Model::ModelManager->instance()->model('Mode');
+            my $remote = $model->remoteValue();
+            throw EBox::Exceptions::Internal("Cannot restore slave machine when master is down: $remote");
+        };
+
+        # Save conf to enable NSS (and/or) PAM
+        $self->_setConf();
         for my $user ($self->users()) {
             $self->initUser($user->{'username'});
         }
         $self->_enforceServiceState();
+    } else {
+        throw EBox::Exceptions::Internal(
+            "Trying to restore configuration of unknown LDAP mode: $mode");
     }
 }
 
@@ -2431,10 +2681,16 @@ sub defaultGroup
 sub authUser
 {
     my ($self, $user, $password) = @_;
-    my $ldap = Net::LDAP->new(EBox::Ldap::LDAPI);
-    $ldap or return undef;
-    my $res = $ldap->bind($self->userDn($user), password => $password);
-    return ($res->{'resultCode'} == 0);
+
+    my $authorized = 0;
+    my $ldap = EBox::Ldap::safeConnect(EBox::Ldap::LDAPI);
+    try {
+        EBox::Ldap::safeBind($ldap, $self->userDn($user), $password);
+        $authorized = 1; # auth ok
+    } otherwise {
+        $authorized = 0; # auth failed
+    };
+    return $authorized;
 }
 
 sub shaHasher
@@ -2513,12 +2769,22 @@ sub defaultPasswordHash
 
 sub _setupSlaveLDAP
 {
-    my ($self) = @_;
+    my ($self, $replicaOnly) = @_;
+
+    # Setup everything by default
+    $replicaOnly = 0 unless defined($replicaOnly);
 
     my ($ldap, $dn) = $self->_connRemoteLDAP();
+
+    # Save LDAP dn in Mode
+    my $model = $self->model('Mode');
+    my $row = $model->row();
+    $row->elementByName('dn')->setValue($dn);
+    $row->store();
+
     $self->_registerHostname($ldap, $dn);
     $self->_getCertificates($ldap, $dn);
-    $self->_setupReplication($dn);
+    $self->_setupReplication($dn, $replicaOnly);
 }
 
 sub _connRemoteLDAP
@@ -2529,17 +2795,9 @@ sub _connRemoteLDAP
     my $remote = $model->remoteValue();
     my $password = $model->passwordValue();
 
-    my $ldap = Net::LDAP->new("ldap://$remote");
-    if (not defined($ldap)) {
-        throw EBox::Exceptions::External(__x("Can't connect to master eBox at {remote}", remote => $remote));
-    }
-
+    my $ldap = EBox::Ldap::safeConnect("ldap://$remote");
     my $dn = baseDn($ldap);
-
-    my $result = $ldap->bind($self->ldap->rootDn($dn), password => $password);
-    if ($result->is_error()) {
-        throw EBox::Exceptions::External(__x("Can't connect to master eBox at {remote} with the provided LDAP password", remote => $remote));
-    }
+    EBox::Ldap::safeBind($ldap, $self->ldap->rootDn($dn), $password);
 
     return ($ldap, $dn);
 }
@@ -2557,7 +2815,7 @@ sub _registerHostname
     );
     my $result = $ldap->search(%args);
     if ($result->count() > 0) {
-        throw EBox::Exceptions::External(__x('A host with the name {host} is already registered in this eBox', host => $hostname));
+        throw EBox::Exceptions::External(__x('A host with the name {host} is already registered in this Zentyal', host => $hostname));
     }
 
     my $apache = EBox::Global->modInstance('apache');
@@ -2607,14 +2865,14 @@ sub _getCertificates
     write_file(CA_DIR . 'masterldapca.pem', $ldapcacert);
     EBox::Sudo::root('ln -sf ' . CA_DIR . 'masterldapca.pem /etc/ldap/ssl/masterldapca.pem');
     EBox::Module::Base::writeConfFileNoCheck('/etc/ldap/ldap.conf',
-            'usersandgroups/ldap.conf.mas',
+            'usersandgroups/ldap-slave.conf.mas',
     );
 
 }
 
 sub _setupReplication
 {
-    my ($self, $remotedn) = @_;
+    my ($self, $remotedn, $replicaOnly) = @_;
 
     my $model = EBox::Model::ModelManager->instance()->model('Mode');
     my $remote = $model->remoteValue();
@@ -2631,18 +2889,21 @@ sub _setupReplication
     EBox::Module::Base::writeFile($ldappass, $password);
 
     $self->_writeLdapConf('replica', $opts);
-    $self->_writeLdapConf('translucent', $opts);
-    $self->_writeLdapConf('frontend', $opts);
 
-    EBox::Module::Base::writeConfFileNoCheck(
-        EBox::Config::tmp() . "slapd-frontend-referrals.ldif",
-        "usersandgroups/slapd-frontend-referrals.ldif.mas",
-        $opts
-    );
-    EBox::Sudo::root('slapadd -F ' . LDAPCONFDIR .
-        "slapd-frontend.d" .  " -b '$remotedn' -l " .
-        EBox::Config::tmp() . "slapd-frontend-referrals.ldif");
-    EBox::Sudo::root("chown -R openldap.openldap /var/lib/ldap-frontend");
+    unless($replicaOnly) {
+        $self->_writeLdapConf('translucent', $opts);
+        $self->_writeLdapConf('frontend', $opts);
+
+        EBox::Module::Base::writeConfFileNoCheck(
+                EBox::Config::tmp() . "slapd-frontend-referrals.ldif",
+                "usersandgroups/slapd-frontend-referrals.ldif.mas",
+                $opts
+                );
+        EBox::Sudo::root('slapadd -F ' . LDAPCONFDIR .
+                "slapd-frontend.d" .  " -b '$remotedn' -l " .
+                EBox::Config::tmp() . "slapd-frontend-referrals.ldif");
+        EBox::Sudo::root("chown -R openldap.openldap /var/lib/ldap-frontend");
+    }
 
     $self->_manageService('start');
     $self->waitSync();
@@ -2660,8 +2921,7 @@ sub waitSync
     }
 
     my $times = 10;
-    while(1) {
-        sleep(3);
+    while (1) {
         my $master_users = $self->listMasterUsers();
         my $replica_users = $self->listReplicaUsers();
 
@@ -2681,6 +2941,7 @@ sub waitSync
         if ($times == 0) {
             throw EBox::Exceptions::Internal(__('Replication failed'));
         }
+        sleep (3);
     }
 }
 
@@ -2733,11 +2994,10 @@ sub listMasterUsers
     my $remote = $model->remoteValue();
     my $password = $model->passwordValue();
 
-    my $ldap = Net::LDAP->new("ldap://$remote");
+    my $ldap = EBox::Ldap::safeConnect("ldap://$remote");
     my $dn = baseDn($ldap);
     my $rootdn = $self->ldap->rootDn($dn);
-
-    $ldap->bind($rootdn, password => $password);
+    EBox::Ldap::safeBind($ldap, $rootdn, $password);
 
     return $self->listUsers($ldap, $dn);
 }
@@ -2749,11 +3009,10 @@ sub listReplicaUsers
     my $model = EBox::Model::ModelManager->instance()->model('Mode');
     my $password = $model->passwordValue();
 
-    my $ldap = Net::LDAP->new("ldap://127.0.0.1:1389");
+    my $ldap = EBox::Ldap::safeConnect('ldap://127.0.0.1:1389');
     my $dn = baseDn($ldap);
     my $rootdn = $self->ldap->rootDn($dn);
-
-    $ldap->bind($rootdn, password => $password);
+    EBox::Ldap::safeBind($ldap, $rootdn, $password);
 
     return $self->listUsers($ldap, $dn);
 }
@@ -2783,10 +3042,10 @@ sub listMasterGroups
     my $password = $model->passwordValue();
 
 
-    my $ldap = Net::LDAP->new("ldap://$remote");
+    my $ldap = EBox::Ldap::safeConnect("ldap://$remote");
     my $dn = baseDn($ldap);
     my $rootdn = $self->ldap->rootDn($dn);
-    $ldap->bind($rootdn, password => $password);
+    EBox::Ldap::safeBind($ldap, $rootdn, $password);
 
     return $self->listGroups($ldap, $dn);
 }
@@ -2799,10 +3058,10 @@ sub listReplicaGroups
 
     my $password = $model->passwordValue();
 
-    my $ldap = Net::LDAP->new("ldap://127.0.0.1:1389");
+    my $ldap = EBox::Ldap::safeConnect('ldap://127.0.0.1:1389');
     my $dn = baseDn($ldap);
     my $rootdn = $self->ldap->rootDn($dn);
-    $ldap->bind($rootdn, password => $password);
+    EBox::Ldap::safeBind($ldap, $rootdn, $password);
 
     return $self->listGroups($ldap, $dn);
 }
@@ -2831,9 +3090,9 @@ sub listMasterSchemas
     my $remote = $model->remoteValue();
     my $password = $model->passwordValue();
 
-    my $ldap = Net::LDAP->new("ldap://$remote");
+    my $ldap = EBox::Ldap::safeConnect("ldap://$remote");
     my $dn = baseDn($ldap);
-    $ldap->bind($self->ldap->rootDn($dn), password => $password);
+    EBox::Ldap::safeBind($ldap, $self->ldap->rootDn($dn), $password);
 
     return $self->listSchemas($ldap);
 }
@@ -2845,10 +3104,10 @@ sub listReplicaSchemas
     my $model = EBox::Model::ModelManager->instance()->model('Mode');
     my $password = $model->passwordValue();
 
-    my $ldap = Net::LDAP->new("ldap://127.0.0.1:$port");
+    my $ldap = EBox::Ldap::safeConnect("ldap://127.0.0.1:$port");
     my $dn = baseDn($ldap);
     my $rootdn = $self->ldap->rootDn($dn);
-    $ldap->bind($rootdn, password => $password);
+    EBox::Ldap::safeBind($ldap, $rootdn, $password);
 
     return $self->listSchemas($ldap);
 }
@@ -2902,6 +3161,10 @@ sub deleteSlave
     my $res = 0;
     try {
         $self->ldap->delete("hostname=$slave," . $self->slavesDn());
+        my $journaldir = $self->_journalsDir . $slave;
+        if (-d $journaldir) {
+            EBox::Sudo::root('rm -rf ' . $journaldir);
+        }
     } otherwise {
         EBox::debug('Error deleting slave: ' . $slave);
         $res = 1;
@@ -2951,4 +3214,42 @@ sub baseDn
     return $dn;
 }
 
+sub _loginShell
+{
+    my ($self) = @_;
+
+    return $self->model('PAM')->login_shellValue();
+}
+
+sub _homeDirectory
+{
+    my ($username) = @_;
+
+    my $home = HOMEPATH . '/' . $username;
+    return $home;
+}
+
+sub _setupNSSPAM
+{
+    my ($self) = @_;
+
+    my @array;
+    my $umask = EBox::Config::configkey('dir_umask');
+    push (@array, 'umask' => $umask);
+
+    $self->writeConfFile(AUTHCONFIGTMPL, 'usersandgroups/acc-ebox.mas',
+               \@array);
+
+    my $enablePam = $self->model('PAM')->enable_pamValue();
+    my @cmds;
+    push (@cmds, 'auth-client-config -a -p ebox');
+
+    unless ($enablePam) {
+        push (@cmds, 'auth-client-config -a -p ebox -r');
+    }
+
+    push (@cmds, 'auth-client-config -t nss -p ebox');
+    EBox::Sudo::root(@cmds);
+}
+
 1;
