@@ -1,4 +1,4 @@
-# Copyright (C) 2009 eBox technologies S.L.
+# Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -22,7 +22,6 @@ use base 'EBox::Model::Composite';
 use strict;
 use warnings;
 
-## eBox uses
 use EBox::Gettext;
 
 # Group: Public methods
@@ -71,7 +70,7 @@ sub _description
 
 sub pageTitle
 {
-    return __('Slave status');
+    return __('Slave Status');
 }
 
 sub menuFolder
