@@ -1,4 +1,4 @@
-# Copyright (C) 2009 eBox technologies S.L.
+# Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -101,7 +101,7 @@ sub precondition
 {
     my ($self) = @_;
     my $users = EBox::Global->modInstance('users');
-    unless ($users->configured() and ($users->mode() eq 'master')) {
+    unless ($users->configured() and ($users->mode() ne 'slave')) {
         $self->{preconFail} = 'notConfigured';
         return undef;
     }
