@@ -1,4 +1,4 @@
-# Copyright (C) 2009 eBox technologies, S.L
+# Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -71,7 +71,7 @@ sub _table
             ['changeView', 'del'],
         'tableDescription' => \@tableHead,
         'menuNamespace' => 'UsersAndGroups/Slaves',
-        'help' => __x('This is a list of those eBox slaves which are subscribed to this eBox.'),
+        'help' => __x('This is a list of those Zentyal slaves which are subscribed to this Zentyal.'),
         'printableRowName' => __('slave'),
         'sortedBy' => 'name',
     };
@@ -90,7 +90,7 @@ sub precondition
 {
     my ($self) = @_;
     my $users = EBox::Global->modInstance('users');
-    unless ($users->configured() and ($users->mode() eq 'master')) {
+    unless ($users->configured() and ($users->mode() ne 'slave')) {
         $self->{preconFail} = 'notConfigured';
         return undef;
     }
@@ -167,7 +167,7 @@ sub row
 
 # Method: removeRow
 #
-#   Override <EBox::Model::DataTable::removeRow> 
+#   Override <EBox::Model::DataTable::removeRow>
 #   to remove a slave
 #
 sub removeRow
