@@ -1,6 +1,4 @@
-# Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
-# Copyright (C) 2006-2008 Warp Networks S.L.
-# Copyright (C) 2009-2010 eBox Technologies S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -135,11 +133,11 @@ sub preconditionFailMsg
             return __x('There are no users at the moment');
         } elsif ($mode eq 'slave') {
             my $master = $users->model('Mode')->remoteValue();
-            return __x('This eBox is configured as slave and there are no users at the moment. You may want to add some in the {openhref}master{closehref}.',
+            return __x('Zentyal is configured as slave and there are no users at the moment. You may want to add some in the {openhref}master{closehref}.',
                        openhref => "<a href='https://$master/ebox/UsersAndGroups/Users'>",
                        closehref => '</a>');
         } elsif ($mode eq 'ad-slave') {
-            return __('This eBox is configured as Windows AD slave and there are no users at the moment. If there are users in your Domain Controller, maybe the synchronization process has failed or has not finished yet.');
+            return __('Zentyal is configured as Windows AD slave and there are no users at the moment. If there are users in your Domain Controller, maybe the synchronization process has failed or has not finished yet.');
         }
     }
 }
